# QA Coverage Dashboard
# Copyright 2012-2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging

from django.shortcuts import get_object_or_404
from django.utils.dateformat import format

from common.utils import JSONResponse
from gaps.models import (CoverageBuild,
                         CoverageData,
                         CoverageProject,
                         CoverageProduct,
                         JenkinsBuild,
                         ProductCoverageObservation)

logger = logging.getLogger('qa_dashboard')


def project(request, name):
    project = get_object_or_404(
        CoverageProject,
        name=name)
    coverage_datas = CoverageData.objects.filter(
        coverage_build__project=project).order_by(
            '-coverage_build__ran_at')
    data = []
    line_coverage_data = {
        'key': 'Line coverage',
        'color': '#DD4814'
    }
    branch_coverage_data = {
        'key': 'Branch coverage',
        'color': '#772953'
    }
    line_coverage_values = []
    branch_coverage_values = []
    for coverage_data in coverage_datas:
        line_coverage_values.append(
            {'y': float(coverage_data.line_coverage),
             'x': format(coverage_data.coverage_build.ran_at, 'U'),})
        branch_coverage_values.append(
            {'y': float(coverage_data.branch_coverage),
             'x': format(coverage_data.coverage_build.ran_at, 'U'),})
    line_coverage_data['values'] = line_coverage_values
    branch_coverage_data['values'] = branch_coverage_values
    data.append(line_coverage_data)
    data.append(branch_coverage_data)
    data = json.dumps(data)
    return JSONResponse(data)

def product(request, name):
    product = get_object_or_404(
        CoverageProduct,
        name=name)
    line_coverage_data = {
        'key': 'Line coverage',
        'color': '#DD4814'
    }
    branch_coverage_data = {
        'key': 'Branch coverage',
        'color': '#772953'
    }
    observations = ProductCoverageObservation.objects.filter(
        product=product).order_by('-timestamp')
    line_coverage_values = []
    branch_coverage_values = []
    # TODO: more Pythonic pls
    for observation in observations:
        line_coverage_values.append(
            {'y': float(observation.line_coverage),
             'x': format(observation.timestamp, 'U'),})
        branch_coverage_values.append(
            {'y': float(observation.branch_coverage),
             'x': format(observation.timestamp, 'U'),})
    line_coverage_data['values'] = line_coverage_values
    branch_coverage_data['values'] = branch_coverage_values
    data = []
    data.append(line_coverage_data)
    data.append(branch_coverage_data)
    data = json.dumps(data)
    return JSONResponse(data)

def product_list(request):
    products = CoverageProduct.objects.all()
    data = []
    for product in products:
        line_coverage_data = {'key': product.name}
        observations = ProductCoverageObservation.objects.filter(
            product=product).order_by('-timestamp')
        line_coverage_values = []
        for observation in observations:
            line_coverage_values.append(
                {'y': float(observation.line_coverage),
                 'x': format(observation.timestamp, 'U'),})
        line_coverage_data['values'] = line_coverage_values
        data.append(line_coverage_data)
    data = json.dumps(data)
    return JSONResponse(data)
