
from django.test import TestCase
from mock import patch, MagicMock

from gaps.util.cu2d import (
    generate_jenkins_job_names,
    refresh_cupstream2distro_config_repo,
    generate_job_configs_from_stack_config,
    UpdateCi,
)


class TestRefreshCupstream2distroConfig(TestCase):

    @patch('os.path.exists')
    @patch('os.chdir')
    @patch('subprocess.call')
    def test_repo_already_exists(
            self,
            subprocess_call_mock,
            os_chdir_mock,
            os_path_exists_mock):
        os_path_exists_mock.return_value = True
        refresh_cupstream2distro_config_repo('/foo/bar')
        subprocess_call_mock.assert_called_with(['bzr', 'pull'])

    @patch('os.path.split')
    @patch('os.chdir')
    @patch('subprocess.call')
    def test_repo_doesnt_exist(
            self,
            subprocess_call_mock,
            os_chdir_mock,
            os_path_split_mock):
        os_path_split_mock.return_value = ('fake_dir', 'fake_repo_name')
        refresh_cupstream2distro_config_repo('/fake/dir')
        subprocess_call_mock.assert_called_with(
            ['bzr',
             'branch',
             'lp:cupstream2distro-config',
             'fake_repo_name']
        )


class TestGetStackListFromFilesystem(TestCase):

    # TODO: more tests pls
    pass


class TestGenerateJobConfigsFromStackConfig(TestCase):

    @patch('gaps.util.cu2d.load_default_cfg')
    @patch('gaps.util.cu2d.load_stack_cfg')
    def test_vanilla(self,
                     load_stack_cfg_mock,
                     load_default_cfg_mock):
        with patch.object(UpdateCi,
                          '__init__',
                          MagicMock(return_value=None)):
            stack_mock = MagicMock(config_filepath='fake_config_filepath')
            generate_job_configs_from_stack_config(
                '/fake/default/config/path',
                stack_mock
            )
        # TODO: get hold of the result and assert

    @patch('gaps.util.cu2d.load_default_cfg')
    @patch('gaps.util.cu2d.load_stack_cfg')
    @patch('logging.error')
    def test_update_ci_process_stack_throws(
            self,
            logger_error_mock,
            load_stack_cfg_mock,
            load_default_cfg_mock):
        with patch.object(UpdateCi,
                          '__init__',
                          MagicMock(
                              return_value=None,
                              side_effect=Exception
                          )):
            with self.assertRaises(Exception):
                stack_mock = MagicMock(config_filepath='fake_config_filepath')
                generate_job_configs_from_stack_config(
                    '/fake/default/config/path',
                    stack_mock
                )

    @patch('gaps.util.cu2d.load_default_cfg')
    @patch('gaps.util.cu2d.load_stack_cfg')
    @patch('logging.error')
    def test_config_failed_to_load(
            self,
            logging_error_mock,
            load_stack_cfg_mock,
            load_default_cfg_mock):
        load_stack_cfg_mock.return_value = None
        stack_mock = MagicMock(config_filepath='fake_config_filepath')
        generate_job_configs_from_stack_config(
            '/fake/default/config/path',
            stack_mock
        )
        logging_error_mock.assert_called()


class TestGenerateJenkinsJobNames(TestCase):

    @patch('gaps.util.cu2d.get_stack_list_from_filesystem')
    @patch('gaps.util.cu2d.generate_job_configs_from_stack_config')
    def test_vanilla(
            self,
            generate_job_configs_from_stack_config,
            get_stack_list_from_filesystem_mock):
        get_stack_list_from_filesystem_mock.return_value = [
            {'config_filepath': 'fake-filepath',
             'pocket': 'fake-pocket',
             'name': 'fake-name'}
        ]
        generate_job_configs_from_stack_config.return_value = [
            {'pocket': 'head',
             'config_filepath': '/home/alesage/workspace/'
             'qa-coverage-dashboard/aggregate-deux/qa_dashboard/..'
             '/gaps/util/cupstream2distro_config/stacks/head/mir.cfg',
             'name': 'fake-jenkins-job-name'}
        ]
        result = generate_jenkins_job_names(
            'fake-default-config-path',
            'fake-stack-dir',
            'fake-jenkins-job-name')
        self.assertEqual(
            ['fake-jenkins-job-name'],
            result
        )
