import logging

from common.management import jenkins_get
from common.models import JenkinsJob
from gaps.models import (
    CoverageBuild,
    CoverageData,
    CoverageProject,
    CoverageProduct,
)
from gaps.util import extractor, jenkins_pull

logger = logging.getLogger('qa_dashboard')


def get_last_build_number_for_jenkins_job(jenkins_job_name):
    logger.debug(
        "Getting the number of the last_build for {}.".format(
            jenkins_job_name))
    try:
        last_coveragedata = CoverageData.objects.filter(
            coverage_build__job__name__exact=jenkins_job_name
        ).order_by(
            '-coverage_build__build_number'
        )[0]
        logger.debug("last build in our database is {}".format(
            last_coveragedata.coverage_build.build_number)
        )
        return [last_coveragedata.coverage_build.build_number]
    except IndexError:
        return [0]


def records(product_name, project_name, jenkins_job_name,
            file_name, just_last_build=False):
    """Adds coverage data to the database

    :param product_name: product to get artifacts from
    :param project name: name of project to get artifact from
    :param jenkins_job_name: job to get artifacts from
    :param file_name: file name of artifact
    :param just_last_build: get just the last build from jenkins
    """
    last_build = get_last_build_number_for_jenkins_job(jenkins_job_name)
    pulled_list = jenkins_pull.url_artifact_list(
        jenkins_job_name,
        file_name,
        last_build[-1],
        just_last_build
    )

    if pulled_list:
        pulled_list.sort()
        if just_last_build:
            pulled_list = [pulled_list[-1]]
        for build_info in pulled_list:
            logger.debug('build info {}'.format(build_info))
            if build_info['url'].endswith('xml'):
                url_file = build_info['url']
                url = '/'.join(build_info['url'].split('/')[:5])
            else:
                url_file = None
                url = build_info['url']
            build_id = build_info['build_id']
            build_ran_at = build_info['ran_at'].replace(tzinfo=None)
            product, product_created = CoverageProduct.objects.get_or_create(
                name=product_name)
            project, project_created = CoverageProject.objects.get_or_create(
                name=project_name)
            # make sure the project is linked to this product
            try:
                CoverageProject.objects.get(name=project_name, product=product)
            except CoverageProject.DoesNotExist:
                project.product.add(product)
            (jenkins_job,
             jenkins_job_created) = JenkinsJob.objects.get_or_create(
                name=jenkins_job_name, url=url)
            build, build_created = CoverageBuild.objects.get_or_create(
                job=jenkins_job,
                failed=False,
                ran_at=build_ran_at,
                build_number=build_id,
                build_description='None',
                project=project
            )
            if url_file:
                jdata = jenkins_get(url_file, as_json=False)
                coverage_info = extractor.JenkinsCoverage(url_file, jdata)
                get_cd, create_cd = CoverageData.objects.get_or_create(
                    url=url_file,
                    name='coverage.xml',
                    xml_file="",  # str(jdata),
                    line_coverage=coverage_info.line_coverage,
                    branch_coverage=coverage_info.branch_coverage,
                    line_count=coverage_info.lines,
                    hit_count=coverage_info.hits,
                    taken_count=coverage_info.taken,
                    total_count=coverage_info.total,
                    coverage_build=build,
                )
            product.refresh_coverage_observations()
