import logging
from django.db import IntegrityError
from django.http import HttpResponse
from django.template import (Context, loader)
from django.shortcuts import get_object_or_404
import gviz_api

from common.bread_crumbs import (BreadCrumb,
                                 BreadCrumbTrail)
from gaps.models import (CoverageProject,
                         CoverageBuild,
                         CoverageData,
                         CoverageProduct,
                         ProductCoverageObservation)

logger = logging.getLogger('qa_dashboard')


@BreadCrumb("Products")
def product_list(request):
    product_list_ = CoverageProduct.objects.all()
    t = loader.get_template('gaps/product_list.html')
    c = Context({'product_list': product_list_,
                 'bread_crumb_trail': BreadCrumbTrail.leading_to(
                     product_list)})
    return HttpResponse(t.render(c))

@BreadCrumb("{name}", parent=product_list, needs=['name'])
def product(request, name):
    product_ = CoverageProduct.objects.get(name=name)
    product_list = CoverageProduct.objects.all()
    project_list = CoverageProject.objects.filter(
        product__name=name)
    t = loader.get_template('gaps/product_detail.html')
    c = Context({'product': product_,
                 'product_list': product_list,
                 'project_list': project_list,
                 'bread_crumb_trail': BreadCrumbTrail.leading_to(
                     product, name=product_.name)})
    return HttpResponse(t.render(c))

@BreadCrumb("Projects")
def project_list(request):
    project_list_ = CoverageProject.objects.all()
    t = loader.get_template('gaps/project_list.html')
    c = Context({'project_list': project_list_,
                 'bread_crumb_trail': BreadCrumbTrail.leading_to(
                     project_list)})
    return HttpResponse(t.render(c))

@BreadCrumb('{name}', parent=project_list, needs=['name'])
def project(request, name):
    project_ = get_object_or_404(
        CoverageProject,
        name=name)
    coverage_datas = CoverageData.objects.filter(
        coverage_build__project=project_).order_by(
            '-coverage_build__ran_at')[:10]
    t = loader.get_template('gaps/project_detail.html')
    c = Context({'project': project_,
                 'coverage_datas': coverage_datas,
                 'bread_crumb_trail': BreadCrumbTrail.leading_to(
                     project, name=project_.name)})
    return HttpResponse(t.render(c))
